PROGRAM AVERAGE_SDF	
   
   IMPLICIT NONE

   INTEGER, PARAMETER :: header_lines = 3
   INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(15,3)
   REAL ( KIND = DP ), PARAMETER :: PI = 3.141592653589793_DP
   CHARACTER ( LEN = 132) :: filename_in
   INTEGER :: IOSTAT, I, IX, IY, IZ, n, nframes, nshell, sumshell, nsup
   INTEGER, ALLOCATABLE, DIMENSION(:,:,:) :: sdf
   REAL ( KIND = DP ) :: vuseful, vshell, r


   ! Get input
   
   WRITE (*,'("Name of SDF file?")')
   READ (*,'(A)') filename_in
   WRITE (*,'("SDF array = [-n:n, -n:n, -n:n]")')
   WRITE (*,'("N?")')
   READ (*,*) n
   WRITE (*,'("Name of frames used in computing SDF?")')
   READ (*,*) nframes
   
   ! Open files
   OPEN ( 11, FILE = filename_in, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Error opening input file'

   ! Allocate array for sdf
   ALLOCATE ( sdf ( -n:n, -n:n, -n:n ) ); sdf = 0

   ! Skip header
   DO I = 1,header_lines
      READ (11,*)
   END DO 
   
   DO IZ = -n, n
      DO IY = -n, n
         READ (11,'(20(I5,1x))') ( sdf(IX, IY, IZ), IX = -n, n )
      END DO
   END DO
   
   ! Close files
   CLOSE (11)
   
   ! Write statistics of SDF array
   WRITE (*,*)
   WRITE (*,*) 'Number of voxels = ', SIZE(sdf)
   WRITE (*,*) 'Maximum value    = ', MAXVAL(sdf)
   WRITE (*,*) 'Sum of values    = ', SUM(sdf)
   WRITE (*,*) '    per frame    = ', REAL(SUM(sdf))/REAL(nframes)
   WRITE (*,*)
   DO I = 0,19
      nsup = COUNT ( sdf > ((REAL(I)/20.)*MAXVAL(sdf)) )
      WRITE (*,*) 'Values > ',I*5,'% = ',nsup,'. Fraction = ',REAL(nsup)/REAL(SIZE(sdf))
   END DO
   WRITE (*,*)
   
   vuseful = (4.*PI/3.)*REAL(n)**3
   WRITE (*,*) 'Useful volume = Number of voxels at r<n = ', vuseful
   WRITE (*,*) 'Average in useful volume = ', REAL(SUM(sdf))/vuseful
   WRITE (*,*) '      per frame          = ', REAL(SUM(sdf))/vuseful/REAL(nframes)
   WRITE (*,*) '  normalizing to maxval  = ', REAL(SUM(sdf))/vuseful/REAL(MAXVAL(sdf))
   WRITE (*,*)
   
   nshell = 0
   sumshell = 0.0
   ! Get number of voxels and counts in external shell
   DO IX = -n, n
      DO IY = -n, n
         DO IZ = -n, n
            r = SQRT( REAL(IX**2 + IY**2 + IZ**2) )
	    IF ( (r > n-10) .AND. (r < n) ) THEN
	       nshell = nshell + 1
	       sumshell = sumshell + sdf(IX,IY,IZ)
	    END IF 
         END DO
      END DO
   END DO
   
   vshell = ((4.*PI/3.)*REAL(n)**3) - ((4.*PI/3.)*REAL(n-10)**3)
   WRITE (*,*) 'Expected shell volume (n-10:n)    = ', vshell
   WRITE (*,*) 'Number of voxels counted in shell = ', nshell
   WRITE (*,*) 'Average in shell = ', REAL(sumshell)/REAL(nshell)
   WRITE (*,*) '      per frame  = ', REAL(sumshell)/REAL(nshell)/REAL(nframes)
   WRITE (*,*) ' normalizing to maxval  = ', REAL(sumshell)/REAL(nshell)/REAL(MAXVAL(sdf))
   WRITE (*,*)
   
END PROGRAM AVERAGE_SDF
