PROGRAM BUTANOL_CONTCAR

! Transforms the refmol.pos file created by sdf into a CONTCAR file
! Specific for nmol Butanol (natoms=15) molecules	
   
   IMPLICIT NONE

   INTEGER, PARAMETER :: header_lines = 1
   INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(15,3)
   INTEGER, PARAMETER :: nmol = 10, natoms = 15
   REAL ( KIND = DP ), PARAMETER :: rmax = 10.0

   CHARACTER (LEN=39) :: dummy
   CHARACTER (LEN=1)  :: ext
   INTEGER :: I, J, IOSTAT, index_o, index_c, index_h, index
   REAL ( KIND = DP ), DIMENSION(3,nmol*4)  :: rc
   REAL ( KIND = DP ), DIMENSION(3,nmol*1)  :: ro
   REAL ( KIND = DP ), DIMENSION(3,nmol*10) :: rh

   
   ! Open files
   OPEN ( 11, FILE = 'refmol.pos', ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Error opening input file'
   OPEN ( 21, FILE = 'refmol.CONTCAR', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Error opening output file'


   ! Skip header
   DO I = 1,header_lines
      READ (11,*)
   END DO
   ! Write header of CONTCAR file 
   WRITE (21,'("Atoms: O C H")')
   WRITE (21,'(3X,F10.5)') 2*rmax
   WRITE (21,'(3X,3(F10.5,2X))') 1.0, 0.0, 0.0
   WRITE (21,'(3X,3(F10.5,2X))') 0.0, 1.0, 0.0
   WRITE (21,'(3X,3(F10.5,2X))') 0.0, 0.0, 1.0
   
   ! Read coordinates now: C C C C O H H H H H H H H H H
   index_c = 0
   index_o = 0
   index_h = 0

   DO I = 1, nmol
      DO J = 1, natoms
      
      SELECT CASE (J)
      
      CASE (1:4)
      
         index_c = index_c + 1
         READ (11,'(A39,3(F7.3))') dummy, rc ( 1:3, index_c )

      CASE (5)
      
         index_o = index_o + 1
         READ (11,'(A39,3(F7.3))') dummy, ro ( 1:3, index_o )

      CASE DEFAULT
      
         index_h = index_h + 1
         READ (11,'(A39,3(F7.3))') dummy, rh ( 1:3, index_h )

      END SELECT
      
      END DO
   END DO 
    
   ! Write CONTCAR file
     
   WRITE (21,'(3(2X,I4))') index_o, index_c, index_h
   WRITE (21,'("Direct")')
   DO I = 1, index_o
      WRITE (21,'(3(1X,F6.3))') ro(1:3,I)/(2.*rmax)
   END DO
   DO I = 1, index_c
      WRITE (21,'(3(1X,F6.3))') rc(1:3,I)/(2.*rmax)
   END DO
   DO I = 1, index_h
      WRITE (21,'(3(1X,F6.3))') rh(1:3,I)/(2.*rmax)
   END DO
   
   ! Close files
   CLOSE (11)
   CLOSE (21)
   
   do i = 1,5
      write (ext,'(i1.1)') i
      open (21,file='refmol'//ext//'.CONTCAR',status='unknown')
      WRITE (21,'("Atoms: O C H")')
      WRITE (21,'(3X,F10.5)') 2*rmax
      WRITE (21,'(3X,3(F10.5,2X))') 1.0, 0.0, 0.0
      WRITE (21,'(3X,3(F10.5,2X))') 0.0, 1.0, 0.0
      WRITE (21,'(3X,3(F10.5,2X))') 0.0, 0.0, 1.0
      WRITE (21,'(3(2X,I4))') 1, 4, 10
      WRITE (21,'("Direct")')
      do j = 1,1 !Oxygen
         index = (i-1)*1 +j
         WRITE (21,'(3(1X,F6.3))') ro(1:3,index)/(2.*rmax)
      end do
      do j = 1,4 !Carbon
         index = (i-1)*4 +j
         WRITE (21,'(3(1X,F6.3))') rc(1:3,index)/(2.*rmax)
      end do
      do j = 1,10 !Hydrogen
         index = (i-1)*10 +j
         WRITE (21,'(3(1X,F6.3))') rh(1:3,index)/(2.*rmax)
      end do
      close(21)
   end do
   
   
END PROGRAM BUTANOL_CONTCAR
