!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module containing the values of chemical, physical, and mathematical constants. 
!
! Constants defined:
!
!	PI
!	ELECTRON_CHARGE
!       NAVOGADRO
!       KE_COULOMB
!       ANGSTROM
!
!  Last change: 23/12/2009
!
!===============================================================================


MODULE CHEMPHYSMATH_CONSTANTS

USE DEFINITIONS, ONLY : DP

IMPLICIT NONE
PUBLIC
   
! .Mathematical constants
REAL ( KIND = DP ), PARAMETER :: PI = 3.141592653589793_DP       !pi
REAL ( KIND = DP ), PARAMETER :: ELECTRON_CHARGE = 1.602E-19_DP  !electron charge in C
REAL ( KIND = DP ), PARAMETER :: NAVOGADRO = 6.0221415E23_DP     !Avogadro's number
REAL ( KIND = DP ), PARAMETER :: KE_COULOMB = 8.987552E9_DP      !Coulomb force constant = 1/(4*pi*epsilon_0) [N*m^2/C^2]
REAL ( KIND = DP ), PARAMETER :: ANGSTROM = 1E-10_DP             !Angstrom = 10^-10 m 

END MODULE CHEMPHYSMATH_CONSTANTS   
