!
! Reads a PDB file and converts it to a CONFIG file for DL_POLY
! CO2 molecules are replaced by rigid CO2 molecules.
!
! 	Reads:
!	1. PDB file
!
!	Writes:
!	1. CONFIG file for DL_POLY
!
!  Last change: 04/03/2011
!

PROGRAM CO2_PDB2DLPOLY	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, ATOM_NAME_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE CONFIG_READWRITE,     ONLY : CONFIG_PDB_READ, CONFIG_DLPOLY_WRITE
   USE FORCEFIELD_VARIABLES, ONLY : molname, atomname
   USE LINEAR_ALGEBRA,       ONLY : NORMALIZE   
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: pdb_file, output_file
   CHARACTER ( LEN = ATOM_NAME_LENGTH ), ALLOCATABLE, DIMENSION(:,:) :: newname
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos, newconfig
   REAL ( KIND = DP ), DIMENSION(3) :: rc, ro1, ro2, d1, d2, dbis, dper
   REAL ( KIND = DP ) :: lco
   INTEGER :: I, J, K, newspecies, nco2
   INTEGER, ALLOCATABLE, DIMENSION(:) :: newmols, newatoms


   ! Get input
   
   WRITE (OUTPUT,'("Name of PDB file?")')
   READ (INPUT,'(A)') pdb_file
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   CALL CONFIG_PDB_READ ( pdb_file, config_pos )

   ! Water modification to TIP4P/2005
   
   WRITE (OUTPUT, *)
   WRITE (OUTPUT, '("Number of species = ",I3)') numspecies
   DO I = 1, numspecies
      WRITE (OUTPUT, '("Species ",I2,".")') I
      WRITE (OUTPUT, '("No. molecules = ",I4,".")') nummols(I)
      WRITE (OUTPUT, '("No. atoms in molecule = ",I3,".")') numatoms(I)
      WRITE (OUTPUT, '("Atom names: ",20(A4,1X))') ( atomname(K,I), K = 1, numatoms(I) )
   END DO 

   WRITE (OUTPUT, '("Which is the number of the species corresponding to CO2? ")', ADVANCE='NO') 
   READ (INPUT, *) nco2
   WRITE (OUTPUT, '("Assumed order of atoms is C O O ")') 
   WRITE (OUTPUT, '("Which is the length of the CO bond? ")', ADVANCE='NO') 
   READ (INPUT, *) lco

   IF ( ALLOCATED(newconfig) )  DEALLOCATE(newconfig)
   IF ( ALLOCATED(newmols) )    DEALLOCATE(newmols)
   IF ( ALLOCATED(newatoms) )   DEALLOCATE(newatoms)
   IF ( ALLOCATED(newname) )    DEALLOCATE(newname)
   newspecies = numspecies
   ALLOCATE ( newmols ( newspecies ) ); newmols = nummols
   ALLOCATE ( newatoms ( newspecies ) )
   newatoms = numatoms; newatoms(nco2) = 3
   ALLOCATE ( newconfig ( 3, maxval(newatoms), maxval(newmols), newspecies ) ) 
   newconfig = 0.0
   ALLOCATE ( newname ( maxval(newatoms), newspecies ) ); newname = ''

   DO I = 1,newspecies
      DO J = 1,newmols(I)
         IF (I == nco2) THEN
            rc = config_pos ( :, 1, J, I )
            d1 = config_pos ( :, 2, J, I ) - config_pos ( :, 1, J, I )
            d2 = config_pos ( :, 3, J, I ) - config_pos ( :, 1, J, I )
            d1 = NORMALIZE(d1)
            d2 = NORMALIZE(d2)
	    dbis = d1 + d2
	    dbis = NORMALIZE(dbis)
	    dper = d1 - d2
	    dper = NORMALIZE(dper)
            ro1 = rc + lco*dper 
            ro2 = rc - lco*dper
	    newconfig(:,1,J,I) = rc;  newname(1,I) = atomname(1,I)
	    newconfig(:,2,J,I) = ro1; newname(2,I) = atomname(2,I)
	    newconfig(:,3,J,I) = ro2; newname(3,I) = atomname(3,I)
	 ELSE
	    newconfig(:,:,J,I) = config_pos(:,:,J,I)
	    newname(:,I) = atomname(:,I)
	 END IF
      END DO	    
   END DO
   
   IF ( ALLOCATED(config_pos) )  DEALLOCATE(config_pos)
   IF ( ALLOCATED(nummols) )     DEALLOCATE(nummols)
   IF ( ALLOCATED(numatoms) )    DEALLOCATE(numatoms)
   IF ( ALLOCATED(atomname) )    DEALLOCATE(atomname)
   numspecies = newspecies
   ALLOCATE ( nummols ( numspecies ) );  nummols = newmols
   ALLOCATE ( numatoms ( numspecies ) ); numatoms = newatoms
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ) 
   config_pos = newconfig
   ALLOCATE ( atomname ( maxval(newatoms), newspecies  ) ); atomname = newname

   CALL CONFIG_DLPOLY_WRITE ( output_file, config_pos )

END PROGRAM CO2_PDB2DLPOLY
