!
! Compares two CONFIG files from DL_POLY and gives 
! minimum, maximum, and average difference between 
! atom coordinates.
!
! 	Reads:
!	1. Two CONFIG file
!
!
!  Last change: 01/10/2010
!

PROGRAM CONFIG_COMPARE	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ
   
   IMPLICIT NONE

   INTEGER :: I, j, k, l, natom, minatom, maxatom
   INTEGER, DIMENSION(4) :: minident, maxident   
   CHARACTER ( LEN = LINE_LENGTH) :: file1, file2, ff_file
   REAL ( KIND = DP ) :: minimum, maximum, tmp
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config1, config2


   ! Get input
   
   WRITE (OUTPUT,'("Name of first file?")')
   READ (INPUT,'(A)') file1
   WRITE (OUTPUT,'("Name of second file?")')
   READ (INPUT,'(A)') file2
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   
   CALL CONFIG_DLPOLY_READ ( file1, config1 )

   CALL CONFIG_DLPOLY_READ ( file2, config2 )
      
   maximum = -1.e6; maxident=(/0,0,0,0/); maxatom = 0
   minimum = +1.e6; minident=(/0,0,0,0/); minatom = 0
   natom = 0

   DO i = 1,numspecies
   DO j = 1,nummols(i)
   DO k = 1,numatoms(i)
      natom = natom + 1
      DO l = 1,3
         tmp = abs( config1(l,k,j,i) - config2(l,k,j,i) )
         IF (tmp < minimum) THEN
            minimum = tmp
            minident(1)=i;minident(2)=j;minident(3)=k;minident(4)=l 
            minatom = natom
         END IF 
         IF (tmp > maximum) THEN
            maximum = tmp
            maxident(1)=i;maxident(2)=j;maxident(3)=k;maxident(4)=l
            maxatom = natom 
         END IF
      END DO
   END DO
   END DO
   END DO 

   PRINT *
   PRINT *, 'Minimum difference= ', minimum, ' for atom ', minatom,' (ident = ',minident,')'  
   PRINT *, 'Maximum difference= ', maximum, ' for atom ', maxatom,' (ident = ',maxident,')'  
   PRINT *
   PRINT *, 'Take care: periodic boundary conditions not considered'
   PRINT * 

END PROGRAM CONFIG_COMPARE
