!
! Reads a CONFIG file from DL_POLY and converts it to a PDB file 
!
! 	Reads:
!	1. CONFIG file for DL_POLY
!
!	Writes:
!	1. PDB file
!
!  Last change: 27/05/2011
!

PROGRAM CONFIG_DLPOLY2PDB

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ, CONFIG_PDB_WRITE
   
   IMPLICIT NONE

   INTEGER :: I
   CHARACTER ( LEN = LINE_LENGTH) :: config_file, output_file, ff_file
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos


   ! Get input
   
   WRITE (OUTPUT,'("Name of CONFIG file?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   

   CALL CONFIG_DLPOLY_READ ( config_file, config_pos )

   CALL CONFIG_PDB_WRITE ( output_file, config_pos )

END PROGRAM CONFIG_DLPOLY2PDB
