!
! Reads a CONFIG file from DL_POLY and converts it to a VASP file 
!
! 	Reads:
!	1. CONFIG file for DL_POLY
!
!	Writes:
!	1. VASP file
!
!  Last change: 15/11/2013
!

PROGRAM CONFIG_DLPOLY2VASP

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE FILES,             ONLY : NEXT_UNIT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH) :: config_file, output_file, ff_file
   
   INTEGER :: I, J, IATOM, ISPEC, UNIT, IOSTAT, number_all
   INTEGER, ALLOCATABLE, DIMENSION(:) :: order_list_spec, order_list_atom, order_list_number
   
   REAL ( KIND = DP ) :: box
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config_pos


   ! Get input
   
   WRITE (OUTPUT,'("Name of CONFIG file?")')
   READ (INPUT,'(A)') config_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Box side length (converter valid only for cubic boxes)?")')
   READ (INPUT,*) box
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   
   number_all = 0
   DO I = 1,numspecies
      number_all = number_all + numatoms(I)
   END DO
   WRITE (OUTPUT,*)
   WRITE ( OUTPUT, "('Number of different molecular species = ',I3)") numspecies
   DO I = 1,numspecies
      WRITE ( OUTPUT, "('Molecule type:',I3,'. Number of atoms in this molecule = ',I4)" ) I,numatoms(I)
   END DO
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("There are ", I5, "d ifferent atoms in the system")') number_all
   
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Give species index and atom index in the order they must appear in the VASP file")')

   ALLOCATE (order_list_spec(number_all), order_list_atom(number_all), order_list_number(number_all) )

   DO I = 1, number_all
      WRITE (OUTPUT,'("Atom ... ",I5,"?")') I
      READ (INPUT,*) order_list_spec(I), order_list_atom(I) 
      IF (order_list_spec(I) > numspecies) STOP 'Invalid species'
      IF (order_list_atom(I) > numatoms(order_list_spec(I))) STOP 'Invalid atom'
      order_list_number(I) = nummols(order_list_spec(I))
   END DO

   CALL CONFIG_DLPOLY_READ ( config_file, config_pos )

   UNIT = NEXT_UNIT ( )	
   OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT,'(''VASP file'')')
   WRITE (UNIT,'(f10.3)') 1.0
   WRITE (UNIT,'(3f10.3)') box,0.0,0.0
   WRITE (UNIT,'(3f10.3)') 0.0,box,0.0
   WRITE (UNIT,'(3f10.3)') 0.0,0.0,box
   WRITE (UNIT,'(10(i3,1x))') (order_list_number(I), I = 1, number_all)
   WRITE (UNIT,'(''Cartesian'')')
   
   DO I = 1, number_all
      ISPEC = order_list_spec(I)
      IATOM = order_list_atom(I)
      DO J = 1, nummols(ISPEC)
         WRITE (UNIT,'(3(1x,f11.8))') config_pos(:, IATOM, J, ISPEC)
      END DO
   END DO
   

END PROGRAM CONFIG_DLPOLY2VASP
