!
! Reads a DL_POLY CONFIG file from DL_POLY and multiplicates the
! number of cells in the three directions.
!
! 	Reads:
!	1. A CONFIG file
!
! 	Writes:
!	1. A CONFIG file
!
!
!  Last change: 12/06/2012
!

PROGRAM CONFIG_MULTIPLE_CELLS	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ, CONFIG_DLPOLY_WRITE
   USE CONFIG_UTILS,      ONLY : CONFIG_JOIN_MOLECULES
   USE CONFIG_VARIABLES,  ONLY : cell
    
   IMPLICIT NONE

   INTEGER :: I, J, K, na, nb, nc, nmult, ICELLA, ICELLB, ICELLC, new_molecule
   INTEGER, DIMENSION(4) :: minident, maxident   
   CHARACTER ( LEN = LINE_LENGTH) :: file1, file2, ff_file
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_new
   REAL ( KIND = DP ), DIMENSION(3) :: displacement_vector


   ! Get input
   
   WRITE (OUTPUT,'("Name of input file?")')
   READ (INPUT,'(A)') file1
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') file2
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Number of cells in a direction?")')
   READ (INPUT,*) na
   WRITE (OUTPUT,'("Number of cells in b direction?")')
   READ (INPUT,*) nb
   WRITE (OUTPUT,'("Number of cells in c direction?")')
   READ (INPUT,*) nc

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   
   CALL CONFIG_DLPOLY_READ ( file1, config )

   CALL CONFIG_JOIN_MOLECULES ( config )
   
   nmult = na * nb * nc
   ALLOCATE ( config_new ( 3, maxval(numatoms), maxval(nummols)*nmult, numspecies ) )

   
   ! Generate all replicas and put positions into config_new
   
   DO ICELLC = 1,nc
   DO ICELLB = 1,nb
   DO ICELLA = 1,na
   
      displacement_vector(1) = float(ICELLA-1)*cell(1,1) + float(ICELLB-1)*cell(2,1) + float(ICELLC-1)*cell(3,1)
      displacement_vector(2) = float(ICELLA-1)*cell(1,2) + float(ICELLB-1)*cell(2,2) + float(ICELLC-1)*cell(3,2)
      displacement_vector(3) = float(ICELLA-1)*cell(1,3) + float(ICELLB-1)*cell(2,3) + float(ICELLC-1)*cell(3,3)

      species_loop: DO I = 1,numspecies
      molecules_loop: DO J = 1,nummols(I)
      
         new_molecule = J + (ICELLA-1)*nummols(I) + (ICELLB-1)*na*nummols(I) + (ICELLC-1)*na*nb*nummols(I)
      
         atoms_loop:   DO K = 1,numatoms(I) 
            
	    config_new(1:3,K,new_molecule,I) = config(1:3,K,J,I) + displacement_vector(1:3)
	    
         END DO atoms_loop
      
      END DO molecules_loop
      END DO species_loop


   END DO
   END DO
   END DO
   
   ! Modify cell parameters
   
   cell(1,:) = cell(1,:) * float(na)
   cell(2,:) = cell(2,:) * float(nb)
   cell(3,:) = cell(3,:) * float(nc)
   
   ! Change number of molecules
   
   DO I = 1,numspecies
      nummols(I) = nummols(I) * nmult
   END DO
   
   CALL CONFIG_DLPOLY_WRITE ( file2, config_new )
      

END PROGRAM CONFIG_MULTIPLE_CELLS
