!
! Reads a config file containing a single methane molecule
! and generates a new config file containing N molecules with
! the C atoms placed at the coordinates given as input.
!

!
!  Last change: 07/12/2012
!

PROGRAM CONFIG_PLACE_METHANE	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_VARIABLES
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ, CONFIG_DLPOLY_WRITE
   
   IMPLICIT NONE

   INTEGER :: I, J, nmolecules
   CHARACTER ( LEN = LINE_LENGTH) :: file1, file2
   REAL ( KIND = DP ) :: minimum, maximum, tmp
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_new
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:) :: shift


   ! Get input
   
   WRITE (OUTPUT,'("Name of methane file?")')
   READ (INPUT,'(A)') file1
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') file2
   WRITE (OUTPUT,'("Number of methane molecules to place?")')
   READ (INPUT,*) nmolecules

   ALLOCATE ( shift ( 3, nmolecules) )
   WRITE (OUTPUT,'("Give XYZ coordinates for each ...")')
   DO I = 1,nmolecules
      READ (INPUT,*) shift(1,I), shift(2,I), shift(3,I)
   END DO 
   
   WRITE (OUTPUT,*)

   numspecies = 1
   IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
   IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
   ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
   ALLOCATE ( atomname(5,numspecies) )
   nummols(1) = 1
   numatoms(1) = 5
   atomname(1,1) = 'C'
   atomname(2,1) = 'H'
   atomname(3,1) = 'H'
   atomname(4,1) = 'H'
   atomname(5,1) = 'H'
   CALL CONFIG_DLPOLY_READ ( file1, config )

   ALLOCATE ( config_new ( 3, 5, nmolecules, 1 ) )

   DO I = 1,nmolecules
      DO J = 1,5
         config_new(1:3, J, I, 1) = config(1:3, J, 1, 1) + shift(1:3, I)
	 print *,i,j,config_new(1:3, J, I, 1)
      END DO
   END DO 
   
   !Change number of molecules and write new config
   
   nummols(1) = nmolecules
   CALL CONFIG_DLPOLY_WRITE ( file2, config_new )

END PROGRAM CONFIG_PLACE_METHANE
