!===============================================================================
! This file is part of MG_DLP_analysis - a set of programs to perform
! different analysis on Molecular Dynamics simulations, mainly intended
! to handle trajectories generated by DL_POLY (HISTORY format).
!
! Copyright (C) 2021 Miguel A. Gonzalez
!
!    MG_DLP_analysis is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    MG_DLP_analysis is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with MG_DLP_analysis.  If not, see <https://www.gnu.org/licenses/>.
!
!===============================================================================
!
! Module to declare the variables related to the force field. 
!
! Variables declared:
!
!	neut,
!       ff_header, 
!       energy_units, 
!       molname(numspecies), 
!       atomname(numatoms,numspecies), 
!       atommass(numatoms,numspecies), 
!       atomcharge(numatoms,numspecies) 
!
!  Last change: 24/12/2009
!
!===============================================================================

MODULE FORCEFIELD_VARIABLES

USE DEFINITIONS, ONLY : LINE_LENGTH, ATOM_NAME_LENGTH, DP

IMPLICIT NONE
PUBLIC

LOGICAL :: neut = .false.

CHARACTER (LEN = LINE_LENGTH) :: ff_header
CHARACTER (LEN = 40) :: energy_units
CHARACTER ( LEN = LINE_LENGTH ), ALLOCATABLE, DIMENSION(:) :: molname
CHARACTER ( LEN = ATOM_NAME_LENGTH ), ALLOCATABLE, DIMENSION(:,:) :: atomname

REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:) :: atommass
REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:) :: atomcharge

END MODULE FORCEFIELD_VARIABLES  

   
