PROGRAM IMIDAZOLIUM_CONTCAR

! Transforms the refmol.pos file created by sdf into a CONTCAR file
! Specific for nmol EMIm (natoms=19) molecules	
   
   IMPLICIT NONE

   INTEGER, PARAMETER :: header_lines = 1
   INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(15,3)
   INTEGER, PARAMETER :: nmol = 40, natoms = 19
   REAL ( KIND = DP ), PARAMETER :: rmax = 10.0

   CHARACTER (LEN=39) :: dummy
   INTEGER :: I, J, IOSTAT, index_n, index_c, index_h
   REAL ( KIND = DP ), DIMENSION(3,nmol*2)  :: rn 
   REAL ( KIND = DP ), DIMENSION(3,nmol*6)  :: rc
   REAL ( KIND = DP ), DIMENSION(3,nmol*11) :: rh

   
   ! Open files
   OPEN ( 11, FILE = 'refmol.pos', ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Error opening input file'
   OPEN ( 21, FILE = 'refmol.CONTCAR', ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   IF ( IOSTAT /= 0 ) STOP 'Error opening output file'


   ! Skip header
   DO I = 1,header_lines
      READ (11,*)
   END DO
   ! Write header of CONTCAR file 
   WRITE (21,'("Atoms: N C H")')
   WRITE (21,'(3X,F10.5)') 2*rmax
   WRITE (21,'(3X,3(F10.5,2X))') 1.0, 0.0, 0.0
   WRITE (21,'(3X,3(F10.5,2X))') 0.0, 1.0, 0.0
   WRITE (21,'(3X,3(F10.5,2X))') 0.0, 0.0, 1.0
   
   ! Read coordinates now: N C N C5 H11
   index_n = 0
   index_c = 0
   index_h = 0

   DO I = 1, nmol
      DO J = 1, natoms
      
      SELECT CASE (J)
      
      CASE (1,3)
      
         index_n = index_n + 1
         READ (11,'(A39,3(F7.3))') dummy, rn ( 1:3, index_n )

      CASE (2,4:8)
      
         index_c = index_c + 1
         READ (11,'(A39,3(F7.3))') dummy, rc ( 1:3, index_c )

      CASE DEFAULT
      
         index_h = index_h + 1
         READ (11,'(A39,3(F7.3))') dummy, rh ( 1:3, index_h )

      END SELECT
      
      END DO
   END DO 
    
   ! Write CONTCAR file
     
   WRITE (21,'(3(2X,I4))') index_n, index_c, index_h
   WRITE (21,'("Direct")')
   DO I = 1, index_n
      WRITE (21,'(3(1X,F6.3))') rn(1:3,I)/(2.*rmax)
   END DO
   DO I = 1, index_c
      WRITE (21,'(3(1X,F6.3))') rc(1:3,I)/(2.*rmax)
   END DO
   DO I = 1, index_h
      WRITE (21,'(3(1X,F6.3))') rh(1:3,I)/(2.*rmax)
   END DO
   
   ! Close files
   CLOSE (11)
   CLOSE (21)
   
END PROGRAM IMIDAZOLIUM_CONTCAR
