PROGRAM sq_XR_standard

! Computes the total S(Q) for X-rays from the partial S(Q)'s,
! calculating the corresponding Q-dependence of the atomic form factors
! XR form factors in file f0_CromerMann.txt
!
! Compute S(Q), not I(Q), i.e. normalized sum of partial S(Q)'s

   IMPLICIT NONE

   INTEGER, PARAMETER :: max_points = 9999 
   INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(15,3)
   INTEGER :: n_types, n_partials, n_skip, n_col, ierr, index, n_points, I, J, K, ISKIP, ITYPE, JTYPE, IOSTAT
   CHARACTER ( LEN = 80), ALLOCATABLE, DIMENSION(:,:) :: sqfile
   CHARACTER ( LEN = 80)                              :: outfile
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:)  :: SQ
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)    :: a, b, f
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)      :: c, conc
   REAL ( KIND = DP ), DIMENSION(max_points)          :: Q, sumf, sumf2, SQlimit, SXQ, NORM
   REAL ( KIND = DP )                                 :: q_min, q_max, qtemp, sqtemp, kvec, total_conc

   ! Get input
   
   WRITE (*,'("Number of atom types?")')
   READ (*,*) n_types
   
   ALLOCATE (conc(n_types), a(4,n_types), b(4,n_types), c(n_types))
   ALLOCATE (sqfile(n_types,n_types), SQ(n_types,n_types,max_points), f(n_types,max_points))
   
   WRITE (*,'("For each atom type, give number or fraction in molecule and atomic form factors")')
   DO I = 1, n_types   
      READ (*,*) conc(I),a(1,I),a(2,I),a(3,I),a(4,I),c(I),b(1,I),b(2,I),b(3,I),b(4,I)
   END DO
     
   n_partials = 0
   DO I = n_types, 1, -1
      n_partials = n_partials + I
   END DO  
   WRITE (*,'("Number of different partial S(Q)s = ", I3)') n_partials
   
   WRITE (*,'("Give file name of each partial S(Q):")')
   DO I = 1, n_types
      DO J = I, n_types   
         WRITE (*,'("Partial between atom types ", I2,"-", I2)') I, J
         READ (*,'(A)') sqfile(I,J)
      END DO
   END DO
   
   WRITE (*,'("Number of lines to skip in each file?")')
   READ (*,*) n_skip  
   WRITE (*,'("Number of column to use?")')
   READ (*,*) n_col
   WRITE (*,'("Output file name for total S(Q)?")')
   READ (*,*) outfile
   
   ! Read data
      
   DO I = 1, n_types
   DO J = I, n_types
      OPEN (11, FILE = sqfile(I,J), ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
      ierr = 0
      DO ISKIP = 1, n_skip
         READ (11,*)
      END DO	
      index = 0
      DO WHILE (ierr == 0)
         READ (11,*,IOSTAT=ierr) qtemp, (sqtemp, K = 2, n_col)
	 IF (ierr == 0) THEN
	    index = index + 1
	    IF (index > max_points) STOP 'Increase max_points'
	    Q(index) = qtemp
	    SQ(I,J,index) = sqtemp
	 END IF
      END DO 
      IF (I == 1) THEN
         n_points = index
	 q_min    = Q(1)
	 q_max    = Q(index)
      ELSE 
         IF (n_points /= index) STOP 'Different number of Q values in files'
	 IF (abs(q_min-Q(1)) > 1.e-3) STOP 'Different Q_min in files' 
	 IF (abs(q_max-Q(index)) > 1.e-3) STOP 'Different Q_max in files' 
      END IF
      CLOSE (11)
   END DO 
   END DO
   
   PRINT *, 'Data read'
   PRINT *, 'Number of points = ', n_points
   PRINT *, 'Q_min = ', q_min
   PRINT *, 'Q_max = ', q_max
 
 
   !Compute Q-dependence of atomic form factors
   
   DO I = 1, n_points
      kvec = Q(I)/(4.0*3.1415926535897931)
      DO ITYPE = 1, n_types
         f(ITYPE,I) = c(ITYPE)
	 DO J = 1,4
	    f(ITYPE,I) = f(ITYPE,I) + a(J,ITYPE) * exp ( -b(J,ITYPE) * kvec * kvec )
	 END DO
      	 f(ITYPE,I) = f(ITYPE,I) * 2.818  !Give f in femtometer
      END DO
   END DO
 

   !Compute asymptotic value as a function of Q
   
   DO I = 1, n_points
      sumf(I) = 0.0
      sumf2(I) = 0.0
      DO ITYPE = 1, n_types
         sumf(I) = sumf(I) + conc(ITYPE)*f(ITYPE,I)
	 sumf2(I) = sumf2(I) + conc(ITYPE)*f(ITYPE,I)**2
      END DO
      SQlimit(I) = sumf2(I) / (sumf(I)**2)
   END DO   
   
   
   !Get normalization factor corresponding to the sum of concentrations
   
   total_conc = sum (conc)
   
   !Compute total S_X(Q) and I(Q)
   
   DO I = 1, n_points
      SXQ(I) = 0.0
      NORM(I) = 0.0
      DO ITYPE = 1, n_types
      DO JTYPE = ITYPE, n_types
         IF (ITYPE == JTYPE) THEN
            SXQ(I) = SXQ(I) + conc(ITYPE)**2 * f(ITYPE,I)**2 *  SQ(ITYPE,JTYPE,I) 
	    NORM(I) = NORM(I) + conc(ITYPE)**2 * f(ITYPE,I)**2  
	 ELSE
            SXQ(I) = SXQ(I) + 2 * conc(ITYPE) * conc(JTYPE) * f(ITYPE,I) * f(JTYPE,I) * SQ(ITYPE,JTYPE,I) 
	    NORM(I) = NORM(I) + 2 * conc(ITYPE) * conc(JTYPE) * f(ITYPE,I) * f(JTYPE,I)  
	 END IF
      END DO	 
      END DO
      SXQ(I) = SXQ(I) / NORM(I)
   END DO   

   OPEN ( 21, FILE = outfile, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
  
   WRITE (21,'("#Q - Atomic form factors - Limit Sum(f^2)/(Sum(f))^2- SXQ - IQ")')
   DO I = 1, n_points
      WRITE (21,*) Q(I), (f(ITYPE,I),ITYPE=1,n_types), SQlimit(I), SXQ(I)
   END DO   

   CLOSE (21)
   
END PROGRAM sq_XR_standard
