PROGRAM SQ_convolution

! To check
! Read a S(Q) file [Q, S(Q)] + a resolution file [Q, sigma(Q)]
! and perform the convolution with a gaussian of sigma = sigma(Q)

   IMPLICIT NONE

   INTEGER, PARAMETER :: max_points = 9999 
   INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(15,3)
   INTEGER :: nsq_skip, nsq_col, nres_skip, nres_col, ierr, index, n_points, &
              nres_points, nres_option, I, J, ISKIP, IOSTAT
   CHARACTER ( LEN = 80)                              :: sqfile, resfile, outfile
   REAL ( KIND = DP ), DIMENSION(max_points)          :: Q, SQ, QRES, RES
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)      :: SQ_CONVOL
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)    :: RES_INTERP
   REAL ( KIND = DP )                                 :: q_min, q_max, qtemp, sqtemp
   REAL ( KIND = DP )                                 :: SIGMA, DQ, QAUX, GAUSS, AREA

   WRITE (*,'("Doubtful!! Better see sqConvol2.exe")')
   WRITE (*,'("Give SQ file name:")')
   READ (*,'(A)') sqfile
   WRITE (*,'("Number of lines to skip in SQ file?")')
   READ (*,*) nsq_skip  
   WRITE (*,'("Number of column to use in SQ file?")')
   READ (*,*) nsq_col
   WRITE (*,*)
   WRITE (*,'("Give resolution file name:")')
   READ (*,'(A)') resfile
   WRITE (*,'("Number of lines to skip in resolution file?")')
   READ (*,*) nres_skip  
   WRITE (*,'("Number of column to use in resolution file?")')
   READ (*,*) nres_col
   WRITE (*,'("Resolution option?")')
   WRITE (*,'("1: dQ (sigma)")')
   WRITE (*,'("2: dQ (fwhm)")')
   WRITE (*,'("3: dQ/Q (sigma)")')
   WRITE (*,'("4: dQ/Q (fwhm)")')
   WRITE (*,'("5: dQ (sigma) if Q<1 & dQ/Q (sigma)")')
   WRITE (*,'("6: dQ (fwhm) if Q<1 & dQ/Q (fwhm)")')
   READ (*,*) nres_option
   WRITE (*,*)
   WRITE (*,'("Output file name for resolution convoluted S(Q)?")')
   READ (*,*) outfile
   WRITE (*,*)
   
   ! Read S(Q) data
      
   OPEN (11, FILE = sqfile, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   ierr = 0
   DO ISKIP = 1, nsq_skip
      READ (11,*)
   END DO	
   index = 0
   DO WHILE (ierr == 0)
      READ (11,*,IOSTAT=ierr) qtemp, (sqtemp, J = 2, nsq_col)
      IF (ierr == 0) THEN
	 index = index + 1
	 IF (index > max_points) STOP 'Increase max_points'
	 Q(index) = qtemp
	 SQ(index) = sqtemp
      END IF
   END DO 
   n_points = index
   q_min    = Q(1)
   q_max    = Q(index)
   CLOSE (11)
   
   ! Read resolution data
      
   OPEN (11, FILE = resfile, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
   ierr = 0
   DO ISKIP = 1, nres_skip
      READ (11,*)
   END DO	
   index = 0
   DO WHILE (ierr == 0)
      READ (11,*,IOSTAT=ierr) qtemp, (sqtemp, J = 2, nres_col)
      IF (ierr == 0) THEN
	 index = index + 1
	 IF (index > max_points) STOP 'Increase max_points'
	 QRES(index) = qtemp
	 RES(index) = sqtemp
      END IF
   END DO 
   nres_points = index
   CLOSE (11)
   
   ! Interpolate resolution data to Q values in S(Q) file
   ALLOCATE ( RES_INTERP(1, n_points) )
   CALL INTERP_LINEAR ( 1, nres_points, QRES, RES, n_points, Q, RES_INTERP )
   
   ! Check interpolation
   !OPEN ( 21, FILE = outfile, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   !WRITE (21,'("#Q -  ResQ")')
   !DO I = 1, n_points
   !   WRITE (21,*) Q(I), RES_INTERP(1,I)
   !END DO   
   !CLOSE (21)
   
   ! Perform convolution with gaussian function of width sigma(Q)
   ALLOCATE ( SQ_CONVOL(n_points) )
   
   DO I = 1, n_points
      SQ_CONVOL(I) = 0.0
   END DO   
   
   DQ = Q(2)-Q(1) !0.01 A-1
   
   DO I = 1, n_points
      SELECT CASE(nres_option)
         CASE(1)
            SIGMA = RES_INTERP(1,I)
         CASE(2)
            SIGMA = RES_INTERP(1,I) / 2.355
         CASE(3)   
            SIGMA = RES_INTERP(1,I) * Q(I)
          CASE(4)   
            SIGMA = RES_INTERP(1,I) * Q(I) / 2.355
          CASE(5)   
            IF (Q(I) < 1.0) THEN
               SIGMA = RES_INTERP(1,I) 
            ELSE
               SIGMA = RES_INTERP(1,I) * Q(I) 
            END IF   
          CASE(6)   
            IF (Q(I) < 1.0) THEN
               SIGMA = RES_INTERP(1,I) / 2.355
            ELSE
               SIGMA = RES_INTERP(1,I) * Q(I) / 2.355
            END IF   
          CASE DEFAULT  
            PRINT *, 'Bad resolution option = ', nres_option
            PRINT *, 'Using nres_option = 1'
            SIGMA = RES_INTERP(1,I)
     END SELECT   
     !PRINT *, I,Q(I), RES_INTERP(1,I), SIGMA
      SIGMA = 1.0
      AREA = 0
      DO J = 1, n_points
         QAUX = Q(J)
         GAUSS = EXP(-(QAUX-Q(I))**2/2.0/SIGMA**2) / SQRT(6.283185307179586) / SIGMA
	 AREA = AREA + GAUSS * DQ
      END DO	 
      DO J = 1, n_points
         QAUX = Q(J)
         GAUSS = EXP(-(QAUX-Q(I))**2/2.0/SIGMA**2) / SQRT(6.283185307179586) / SIGMA
	 SQ_CONVOL(J) = SQ_CONVOL(J) + SQ(I) * GAUSS * DQ / AREA
      END DO
      !PRINT *, I, AREA
   END DO   
   
   OPEN ( 21, FILE = outfile, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (21,'("#Q -  S(Q) convoluted with resolution")')
   DO I = 1, n_points
      WRITE (21,*) Q(I), SQ_CONVOL(I)
   END DO   
   CLOSE (21)
   
 
END PROGRAM SQ_convolution
   

