PROGRAM TPYNO3_POPULATIONS

! Test program to determine univocally the starting positions	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,      ONLY : CONFIG_CART_2_FRACT
   USE CONFIG_VARIABLES,  ONLY : cell
   USE FORCEFIELD_VARIABLES,  ONLY : atomname
   USE CHEMPHYSMATH_CONSTANTS
   USE LINEAR_ALGEBRA,       ONLY : NORMALIZE, CROSS_PRODUCT   
   IMPLICIT NONE

   INTEGER :: I, i1, j1, k1, i2, j2, k2, l, neighbours, iswap(1), itemp, iswap1, atom_sel
   INTEGER, PARAMETER :: maxneighbours = 9
   INTEGER, DIMENSION(3,maxneighbours) :: neighbour_list  
   CHARACTER ( LEN = 3) , DIMENSION(6) :: molfirstneighbour_list  
   CHARACTER ( LEN = LINE_LENGTH) :: file, ff_file
   REAL ( KIND = DP ) :: theta
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_fract
   REAL ( KIND = DP ), DIMENSION(maxneighbours) :: neighbour_dist  
   REAL ( KIND = DP ), DIMENSION(6) :: molfirstneighbour_dist, molfirstneighbour_angle  
   REAL ( KIND = DP ), DIMENSION(3) :: sdist, ref_odd, ref_even, cnvector, sign  
   INTRINSIC MINLOC


   ! Get input
   
   WRITE (OUTPUT,'("Name of CONFIG file?")')
   READ (INPUT,'(A)') file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 

   CALL CONFIG_DLPOLY_READ ( file, config )
   CALL CONFIG_CART_2_FRACT (config, config_fract)
      
   !Reference vector: C-N from first and second pyridinium molecules in CONFIG
      
   ref_odd =  config(1:3, 1, 1, 1) - config(1:3, 4, 1, 1)
   ref_even =  config(1:3, 1, 2, 1) - config(1:3, 4, 2, 1)
   ref_odd = NORMALIZE(ref_odd)
   ref_even = NORMALIZE(ref_even)
       
   atom_sel = 1  !select reference atom in pyridinium (NA=1) 
    
   DO i1 = 1,1         !loop over pyridiniums only
   DO j1 = 1,128,2     !nummols(i)
         
      cnvector = config(1:3, atom_sel, j1, i1) - config(1:3, atom_sel+3, j1, i1)
      cnvector = NORMALIZE(cnvector)
      
      theta = ACOS (DOT_PRODUCT(ref_odd,cnvector))
      theta = theta*180./pi
      sign = CROSS_PRODUCT(ref_odd,cnvector)
      if (sign(3) < 0.0) theta = 360.0-theta
      
      WRITE (OUTPUT,'(I3,5x,F6.1)') j1, theta  
   
   END DO
   END DO 

END PROGRAM TPYNO3_POPULATIONS

