PROGRAM TPYNO3_POPULATIONS

! Test program to determine univocally the starting positions	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,      ONLY : CONFIG_CART_2_FRACT
   USE CONFIG_VARIABLES,  ONLY : cell
   USE FORCEFIELD_VARIABLES,  ONLY : atomname
   USE CHEMPHYSMATH_CONSTANTS
   USE LINEAR_ALGEBRA,       ONLY : NORMALIZE, CROSS_PRODUCT   
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE FILES,                  ONLY : NEXT_UNIT

   IMPLICIT NONE

   INTEGER :: I, i1, j1, k1, i2, j2, k2, kopposite, l, iswap(1), itemp, iswap1, atom_sel, atom_opp
   INTEGER :: nspecies, nfirst, nlast, njump, nframes, iframe, iangle, total, unit, iostat
   INTEGER, DIMENSION(6) :: pop
   INTEGER, DIMENSION(-31:331) :: final_dist
   CHARACTER ( LEN = LINE_LENGTH) :: file, ff_file, traject_file, output_file
   CHARACTER ( LEN = 1) :: name
   REAL ( KIND = DP ) :: theta, rmin, rx, ry, rz, r, dot, diff, thetaw, weight
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_fract
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr
   REAL ( KIND = DP ), DIMENSION(3) :: sdist, sign, ccvector, cn  
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:) :: cnvector, pn  
   REAL ( KIND = DP ), DIMENSION(-31:331) :: final_dist_norm


   ! Get input
   
   WRITE (OUTPUT,'("Name of CONFIG file?")')
   READ (INPUT,'(A)') file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Which is the number of the pyridinium species in FIELD (normally 1)?")')
   READ (INPUT,*) nspecies
   WRITE (OUTPUT,'("Number of first molecule to use?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Number of last molecule to use?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Jump between molecules to use?")')
   READ (INPUT,*) njump
   WRITE (OUTPUT,*)
   ALLOCATE ( cnvector(3,nlast) ); cnvector = 0.0
   ALLOCATE ( pn(6,nlast) ); pn = 0.0
   final_dist = 0
   final_dist_norm = 0.0

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 

   CALL CONFIG_DLPOLY_READ ( file, config )
   CALL CONFIG_CART_2_FRACT (config, config_fract)
      
   !Reference vector: C-N from first and second pyridinium molecules in CONFIG
      
   atom_sel = 1  !select reference atom in pyridinium (NA=1) 
   atom_opp = 4  !select opposite atom to reference atom in 
    
   DO i1 = nspecies,nspecies         !loop over pyridiniums only
   DO j1 = nfirst,nlast,njump        !nummols(i)
         
      cnvector(1:3,j1) = config(1:3, atom_sel, j1, i1) - config(1:3, atom_opp, j1, i1)
      cnvector(1:3,j1) = NORMALIZE(cnvector(1:3,j1))
      
      DO k1 = 1,6     !ring atoms
      
         kopposite = k1 + 3
	 if (kopposite > 6) kopposite = kopposite - 6
         ccvector = config(1:3, k1, j1, i1) - config(1:3, kopposite, j1, i1)
         ccvector = NORMALIZE(ccvector)
	 dot = DOT_PRODUCT(cnvector(1:3,j1),ccvector)
	 if (dot > 1.0) dot=1.0
	 if (dot < -1.0) dot=-1.0
         theta = ACOS (dot)
         theta = theta*180./pi
         sign = CROSS_PRODUCT(cnvector(1:3,j1),ccvector)
         if (sign(3) < 0.0) theta = 360.0-theta
      
         !Now search closer O, S or N atom to hydrogen to determine Pn
	 
	 rmin = 3.
         DO i2 = 1,numspecies
         DO j2 = 1,nummols(i2)
         DO k2 = 1,numatoms(i2)

            IF ((I1==I2).AND.(J1==J2)) CYCLE
	 
	    ! Obtain distance between atoms
			
	    sdist = config_fract(1:3, K1+6, J1, I1) - config_fract(1:3, K2, J2, I2)
	    sdist = sdist - ANINT(sdist)
            rx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
            ry = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
            rz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
            r = SQRT( rx*rx + ry*ry + rz*rz )
	    
            IF (r < rmin) THEN
	       IF ( (atomname(k2,i2)(1:1) == 'O') .OR. (atomname(k2,i2)(1:1) == 'S') .OR. (atomname(k2,i2)(1:1) == 'N') ) THEN
	          rmin = r
		  name = atomname(k2,i2)(1:1)
	       ENDIF   
            END IF 
	 
         END DO
         END DO
         END DO 
         
	 IF (name == 'O') THEN
	    IF (rmin < 2.2) THEN 
	       pn(1,j1)= theta 
	    ELSE 
	       pn(2,j1) = theta 
	    END IF   
	 END IF
	 IF (name == 'S') THEN
	    IF (rmin < 2.8) THEN 
	       pn(3,j1)= theta 
	    ELSE 
	       pn(4,j1) = theta 
	    END IF   
	 END IF
	 IF (name == 'N') THEN
	    IF (rmin < 2.75) THEN 
	       pn(5,j1)= theta 
	    ELSE 
	       pn(6,j1) = theta
	    END IF    
	 END IF
	 
      END DO
      
      WRITE (OUTPUT,'(I3,5x,6(F5.1,1x))') j1,pn(:,j1)
   
   END DO
   END DO 

   ! Get further input
   
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("Name of HISTORY file?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file for HISTORY (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of frames in trajectory file?")')
   READ (INPUT,*) nframes
   WRITE (OUTPUT,'("Name of output file?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 
   
   ! Read trajectory 
   
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)

   
   pop = 0
   
   frames_loop: DO IFRAME = 1,nframes

      ! For each molecule determine angle with respect to the same molecule in CONFIG
      ! and then assign to the right population: P1(O1), P2(O2), P3(S1), P4(S2), P5(N1), P6(N2).
   
      DO i1 = nspecies,nspecies         !loop over pyridiniums only
      DO j1 = nfirst,nlast,njump        !nummols(i)
         
         cnvector(1:3,j1) = config(1:3, atom_sel, j1, i1) - config(1:3, atom_opp, j1, i1)
         cnvector(1:3,j1) = NORMALIZE(cnvector(1:3,j1))

         cn = rrr(1:3, atom_sel, j1, i1, IFRAME) - rrr(1:3, atom_opp, j1, i1, IFRAME)
         cn = NORMALIZE(cn)
	 dot = DOT_PRODUCT(cnvector(1:3,j1),cn)
	 if (dot > 1.0) dot=1.0
	 if (dot < -1.0) dot=-1.0
         theta = ACOS (dot)
         theta = theta*180./pi
         sign = CROSS_PRODUCT(cnvector(1:3,j1),cn)
         if (sign(3) < 0.0) theta = 360.0-theta
      
         ! Check to which population corresponds this angle
	 if (theta > 330.) theta = theta - 360.0
	 DO L = 1,6
	    IF ((theta > pn(L,j1)-30.0).AND.(theta < pn(L,j1)+30.0)) THEN
	       pop(L) = pop(L) + 1
	       diff = theta - pn(L,j1)
	       iangle = INT (60*(L-1)+diff)
	       if ((iangle < -31).OR.(iangle > 331)) STOP 
	       final_dist(iangle) = final_dist(iangle)+1
	       IF (ABS(theta) < 0.01) THEN
	          thetaw = 0.001
	       ELSE
	          thetaw = theta
	       END IF	  
	       weight = ABS(1./sin(thetaw*pi/180.))
	       final_dist_norm(iangle) = final_dist_norm(iangle) + weight
	    ENDIF   
	 END DO
   
      END DO
      END DO 

   END DO frames_loop

   total = SUM(pop)

    ! Write population distribution

    UNIT = NEXT_UNIT ( )	
    OPEN ( UNIT, FILE = output_file, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
    WRITE (UNIT,'(''# Py populations'')')
    WRITE (UNIT,'(''# Number of elements counted = '',I6)') total
    DO L = 1,6
       WRITE (UNIT,'(''# % population '',I1,'' = '',F6.2)') L, 100*REAL(pop(L))/REAL(total)
    END DO
    WRITE (UNIT,'(''# Angle - Hits'')')
    DO L = -31,331
       WRITE (UNIT,"(1x,I3,3x,I6,3x,F12.2)") l, final_dist(l), final_dist_norm(l)
    END DO
    CLOSE (UNIT)


END PROGRAM TPYNO3_POPULATIONS

