PROGRAM TPYNO3_POSITIONS

! Test program to determine univocally the starting positions	

   USE DEFINITIONS,       ONLY : LINE_LENGTH, DP
   USE IO_UNITS,          ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE FORCEFIELD_READ,   ONLY : FIELD_DLPOLY_READ
   USE CONFIG_READWRITE,  ONLY : CONFIG_DLPOLY_READ
   USE CONFIG_UTILS,      ONLY : CONFIG_CART_2_FRACT
   USE CONFIG_VARIABLES,  ONLY : cell
   USE FORCEFIELD_VARIABLES,  ONLY : atomname
   USE CHEMPHYSMATH_CONSTANTS
   
   IMPLICIT NONE

   INTEGER :: I, i1, j1, k1, i2, j2, k2, l, neighbours, iswap(1), itemp, iswap1
   INTEGER, PARAMETER :: maxneighbours = 9
   INTEGER, DIMENSION(3,maxneighbours) :: neighbour_list  
   CHARACTER ( LEN = 3) , DIMENSION(6) :: molfirstneighbour_list  
   CHARACTER ( LEN = LINE_LENGTH) :: file, ff_file
   REAL ( KIND = DP ) :: limit_search, temp, rx, ry, rz, r, rax, ray, raz, ra, rbx, rby, rbz, rb, theta
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:) :: config, config_fract
   REAL ( KIND = DP ), DIMENSION(maxneighbours) :: neighbour_dist  
   REAL ( KIND = DP ), DIMENSION(6) :: molfirstneighbour_dist, molfirstneighbour_angle  
   REAL ( KIND = DP ), DIMENSION(3) :: sdist  
   INTRINSIC MINLOC


   ! Get input
   
   WRITE (OUTPUT,'("Name of CONFIG file?")')
   READ (INPUT,'(A)') file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Limit to search neighbours?")')
   READ (INPUT,*) limit_search
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 

   CALL CONFIG_DLPOLY_READ ( file, config )
   CALL CONFIG_CART_2_FRACT (config, config_fract)
      
   DO i1 = 1,1
   DO j1 = 1,128,2     !nummols(i)
   DO k1 = 7,7       !1,numatoms(i)
      
      neighbours = 0
      
      DO i2 = 1,numspecies
      DO j2 = 1,nummols(i2)
      DO k2 = 1,numatoms(i2)

         IF ((I1==I2).AND.(J1==J2)) CYCLE
	 
	 ! Obtain distance between atoms
			
	 sdist = config_fract(1:3, K1, J1, I1) - config_fract(1:3, K2, J2, I2)
	 sdist = sdist - ANINT(sdist)
         rx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
         ry = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
         rz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
         r = SQRT( rx*rx + ry*ry + rz*rz )
	 
         IF (r < limit_search) THEN
	    IF ( (atomname(k2,i2)(1:1) == 'O') .OR. (atomname(k2,i2)(1:1) == 'S') .OR. (atomname(k2,i2)(1:1) == 'N') ) THEN
               neighbours = neighbours + 1
               IF (neighbours > maxneighbours) STOP 'Too many neighbours' 
               neighbour_list(1,neighbours) = i2 
               neighbour_list(2,neighbours) = j2 
               neighbour_list(3,neighbours) = k2 
	       neighbour_dist(neighbours) = r
	       !WRITE (OUTPUT,'(I1,1x,I3,1x,I2," - ",I1,1x,I3,1x,I2," :",F5.3)') i1,j1,k1,i2,j2,k2,r
	    ENDIF   
         END IF 

      END DO
      END DO
      END DO 

      IF (neighbours==0) THEN
         molfirstneighbour_list(k1-7+1) = ' '
	 molfirstneighbour_dist(k1-7+1) = 0.0
	 molfirstneighbour_angle(k1-7+1) = 0.0
         CYCLE
      END IF

      ! Sort array with increasing distances
      
      !WRITE (OUTPUT,'("Atom ",I1,1x,I3,1x,I2,1x,"(",A2,") . Number of neighbours",I2)') i1,j1,k1,atomname(k1,i1),neighbours
     
      DO L = 1,neighbours-1
     
         ISWAP = MINLOC(neighbour_dist(L:neighbours))
         ISWAP1=ISWAP(1)+L-1
         IF(ISWAP1.NE.L) THEN
            TEMP=neighbour_dist(L)
            neighbour_dist(L)=neighbour_dist(ISWAP1)
            neighbour_dist(ISWAP1)=TEMP
            ITEMP=neighbour_list(1,L)
            neighbour_list(1,L)=neighbour_list(1,ISWAP1)
            neighbour_list(1,ISWAP1)=ITEMP
            ITEMP=neighbour_list(2,L)
            neighbour_list(2,L)=neighbour_list(2,ISWAP1)
            neighbour_list(2,ISWAP1)=ITEMP
            ITEMP=neighbour_list(3,L)
            neighbour_list(3,L)=neighbour_list(3,ISWAP1)
            neighbour_list(3,ISWAP1)=ITEMP
         ENDIF
	
      END DO
      
      !DO L = 1,neighbours
      !   WRITE (OUTPUT,'(I1,1x,I3,1x,I2," - ",I1,1x,I3,1x,I2," :",F5.3)') i1,j1,k1,neighbour_list(1,L),neighbour_list(2,L),&
      !	                                                                & neighbour_list(3,L),neighbour_dist(L)
      !END DO	
      !WRITE (OUTPUT,*) 
      
      WRITE (OUTPUT,'(I3,1x,A3,":")') j1, atomname(k1,i1)
      WRITE (OUTPUT,'(9(A3,1x,F5.3,",",2x))') (atomname(neighbour_list(3,L),neighbour_list(1,L)),neighbour_dist(L), L=1,1) 
      !WRITE (OUTPUT,*)
      
      molfirstneighbour_list(k1-7+1) = atomname(neighbour_list(3,1),neighbour_list(1,1))
      molfirstneighbour_dist(k1-7+1) = neighbour_dist(1)
      
      !compute N(C)H...(O,N,S) angle

      sdist = config_fract(1:3, K1, J1, I1) - config_fract(1:3, K1-6, J1, I1)
      sdist = sdist - ANINT(sdist)
      rax = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
      ray = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
      raz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
      ra = SQRT( rax*rax + ray*ray + raz*raz )

      sdist = config_fract(1:3, neighbour_list(3,1), neighbour_list(2,1), neighbour_list(1,1)) - config_fract(1:3, K1, J1, I1) 
      sdist = sdist - ANINT(sdist)
      rbx = sdist(1)*cell(1,1) + sdist(2)*cell(2,1) + sdist(3)*cell(3,1)
      rby = sdist(1)*cell(1,2) + sdist(2)*cell(2,2) + sdist(3)*cell(3,2)
      rbz = sdist(1)*cell(1,3) + sdist(2)*cell(2,3) + sdist(3)*cell(3,3)
      rb = SQRT( rbx*rbx + rby*rby + rbz*rbz )
      
      theta = ACOS ( (rax*rbx+ray*rby+raz*rbz)/ra/rb ) * 180.0 / PI
      molfirstneighbour_angle(k1-7+1) = theta
      
   END DO
   !WRITE (OUTPUT,'("# Molecule ",I3," : ",2x,6(A5,1x))') j1, molfirstneighbour_list
   !WRITE (OUTPUT,'(" ",10x,I3,5x,6(F5.3,1x))') j1,molfirstneighbour_dist
   !WRITE (OUTPUT,'(" ",10x,I3,5x,6(F5.1,1x))') j1,molfirstneighbour_angle
   END DO
   END DO 

END PROGRAM TPYNO3_POSITIONS

