!
! Reads the file(s) produced by the C program voro++ and
! analyse them to produce some histograms and do some other
! specific analysis between clusters (defined as a molecule +
! all the molecules in neighbouring Voronoi volumes)
!
!  Last change: 28/04/2013
!

PROGRAM VORONOI_ORIENTATION_ANALYSIS	
   

   USE DEFINITIONS,            ONLY : LINE_LENGTH, DP
   USE IO_UNITS,               ONLY : INPUT, OUTPUT
   USE FILES,                  ONLY : NEXT_UNIT
   USE PARSING,                ONLY : GET_LINE, GET_WORD, GET_INTEGER, GET_REAL, WORD, WRDLEN, EMPTY, PUSH_UNIT, POP_UNIT
   USE CHEMPHYSMATH_CONSTANTS, ONLY : PI
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES,   ONLY : frame_cell, average_cell_volume, pbc_jumps
   USE FORCEFIELD_READ,        ONLY : FIELD_DLPOLY_READ
   USE FORCEFIELD_VARIABLES,   ONLY : atomname
   USE TRAJECTORY_READWRITE,   ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,       ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES, TRAJECT_CART_2_FRACT, TRAJECT_FRACT_2_CART
   USE ORIENTATION_UTILS,      ONLY : DEFINE_MOLECULAR_ORIENTATION_GEN
   USE LINEAR_ALGEBRA,         ONLY : NORMALIZE

   IMPLICIT NONE

   LOGICAL                            :: normalize_maximum = .TRUE. 
   CHARACTER ( LEN = LINE_LENGTH) :: voro_file, vol_file, traject_file, ff_file, output_file_1, output_file_2, output_file_3
   CHARACTER ( LEN = 4)           :: frame_number
   CHARACTER ( LEN = 2)           :: index1_name, index2_name
   CHARACTER ( LEN = 1)           :: answer, iface_number
   INTEGER                        :: UNIT, IOSTAT, I, J, K, IFRAME, nfirst, nlast, nframes, nmolecules, index, n_volumes
   INTEGER                        :: UNIT1, UNIT2, UNIT3, UNIT4, UNIT5, UNIT6, UNIT7, UNIT8, UNIT9
   INTEGER                        :: number_points, number_th, number_r, bin, binth, binr, IFACE, IAUX
   INTEGER                        :: vertices, edges, faces, index_face
   INTEGER, PARAMETER             :: max_print = 1
   INTEGER, DIMENSION(3)          :: triademol
   INTEGER, DIMENSION(25)         :: neighbour
   INTEGER, DIMENSION(5)          :: nprinted
   REAL ( KIND = DP ), DIMENSION(3)   :: mol_center, sdist, tmp_axis
   REAL ( KIND = DP ), DIMENSION(3,3) :: mol_axes
   REAL ( KIND = DP )             :: x, y, z, total_weight, volume
   LOGICAL                        :: end = .FALSE.
 
   REAL ( KIND = DP ) :: rmax, dr, dtheta, rx, ry, rz, r, dot, xa, xb, xx, shell_volume, n_ideal, &
                         rdf, theta3, sinth, out1, out2, out3, out4, norm_value_1, norm_value_2, &
			 norm_value_3, norm_value_4, norm_value_5, norm_value_6, norm_value_7, norm_value_8
  
   INTEGER, ALLOCATABLE, DIMENSION(:,:)                  :: triade, orient_nummols, norm1, norm2
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:)         :: weightmol
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:)       :: molecule, weight
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: rrr, sss, traj_mol_center, traj_mol_axes
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:)     :: ocf, hist_distth, hist_thdist, hist_distth_sa, hist_thdist_sa, &
                                                            hist_distth_2, hist_thdist_2, hist_distth_sa_2, hist_thdist_sa_2
 
   ! Get input for Voronoi files
   
   WRITE (OUTPUT,'("VORONOI INPUT:")')
   WRITE (OUTPUT,'("Voronoi file name (common part)? Assumes files named file_nameXXXX.xyz.vol")')
   READ (INPUT,'(A)') voro_file
   WRITE (OUTPUT,'("First file to use?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Last file to use?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Number of molecules in file?")')
   READ (INPUT,*) nmolecules
   WRITE (OUTPUT,*)
   WRITE (OUTPUT,'("TRAJECTORY INPUT:")')
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,*)

   ! Get input for orientational distribution
   
   WRITE (OUTPUT,'("Name of output file for orientational correlation function?")')
   READ (INPUT,'(A)') output_file_1
   WRITE (OUTPUT,'("Maximum distance to compute orientational correlation function?")')
   READ (INPUT,*) rmax
   WRITE (OUTPUT,'("Delta r value to use in o.c.f. grid?")')
   READ (INPUT,*) dr
   WRITE (OUTPUT,'("Name of output file for (r,angle) histogram?")')
   READ (INPUT,'(A)') output_file_2
   WRITE (OUTPUT,'("Delta th value to use in (r,angle) grid?")')
   READ (INPUT,*) dtheta
   WRITE (OUTPUT,'("Name of output file for (angle,r) histogram?")')
   READ (INPUT,'(A)') output_file_3
   WRITE (OUTPUT,'("Normalize so that maximum of each distribution = 9.999?")')
   READ (INPUT,'(A)') answer
   IF ( (answer /= "Y") .AND. (answer /= "y") ) normalize_maximum = .FALSE.
   WRITE (OUTPUT,*)

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF (numspecies > 1) STOP 'Not ready for more than 1 species'
      IF ( ALLOCATED(nummols) )  DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"?")') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule?")')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
      WRITE ( OUTPUT, '("System information:")' ) 
      WRITE ( OUTPUT, '("Number of different species = ",I3)') numspecies
      IF (numspecies > 1) STOP 'Not ready for more than 1 species'
      DO I = 1,numspecies
         WRITE ( OUTPUT, "('Molecule ',I3,' contains ',I3,' atoms')" ) I,numatoms(I)
      END DO
      WRITE ( OUTPUT, * )
   END IF 

   ! Give desired weights to calculate the COM of each molecule & 3 atoms to define the molecular orientation
   ! Axis 1: Along vector atom1-->atom2
   ! Axis 2: Perpendicular to 1 and in plane formed by the three atoms
   ! Axis 3: Axis 1 x Axis 2
   
   ALLOCATE ( weight ( maxval(numatoms), numspecies ) );   weight = 0.0
   ALLOCATE ( weightmol ( maxval(numatoms) ) );   weightmol = 0.0
   ALLOCATE ( triade ( 3, numspecies ) );   triade = 0
   ALLOCATE ( molecule ( 3, maxval(numatoms) ) ); molecule = 0.0
   DO I = 1,numspecies
      WRITE (OUTPUT,'("Molecule type ",I3,": Number of atoms = ",I4)') I,numatoms(I)
      WRITE (OUTPUT,'("Give weights of all atoms ")')
      READ ( INPUT, * ) ( weight(J,I), J=1,numatoms(I))
      IF (numatoms(I) >= 3) THEN
         WRITE (OUTPUT,'("Give numbers of 3 atoms to define the molecular orientation")')
	 READ ( INPUT, * ) (triade(J,I), J=1,3)
      END IF
   END DO

   nframes = nlast 
   
   ! Read trajectory and unwrap and re-join molecules

   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ (traject_file, nframes, rrr)
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP (nframes, rrr)  
   CALL TRAJECT_JOIN_MOLECULES (nframes, rrr)

   ! Now calculate trajectory of molecular centers and axes
     
   IF (ALLOCATED(traj_mol_center)) DEALLOCATE(traj_mol_center)
   IF (ALLOCATED(traj_mol_axes))   DEALLOCATE(traj_mol_axes)
   ALLOCATE ( traj_mol_center ( 3, 1, maxval(nummols), numspecies, nframes ) ); traj_mol_center = 0.0 
   ALLOCATE ( traj_mol_axes ( 3, 3, maxval(nummols), numspecies, nframes ) );   traj_mol_axes = 0.0

   DO IFRAME = 1, nframes
      DO I = 1, numspecies
         DO J = 1, nummols(I)
            molecule(1:3,:) = rrr ( 1:3, :, J, I, IFRAME)
            weightmol = weight(:,I)
            triademol = triade(:,I)
            IF (numatoms(I) >= 3) THEN
	       CALL DEFINE_MOLECULAR_ORIENTATION_GEN (molecule, weightmol, triademol, mol_center, mol_axes)
            ELSE
               mol_center = 0.0; total_weight = 0.0
               DO K = 1, numatoms(I)
                  mol_center(1:3) = mol_center(1:3) + weight(K,I) * molecule(1:3,K)
                  total_weight = total_weight + weight(K,I)
               END DO 
               mol_center = mol_center / total_weight
               IF (numatoms(I) == 2) THEN
                  ! Define first axis as bond between both atoms
                  mol_axes(1:3,1) = molecule(1:3,2) - molecule(1:3,1)	 
                  mol_axes(1:3,1) = NORMALIZE( mol_axes(1:3,1) )
               ELSE 
                  ! Use the cartesian axes
                  mol_axes(1:3,1) =  (/ 1.0d0, 0.0d0, 0.0d0 /)
               END IF
               mol_axes(1:3,2) =  (/ 0.0d0, 1.0d0, 0.0d0 /)
               mol_axes(1:3,3) =  (/ 0.0d0, 0.0d0, 1.0d0 /)
            END IF            
	    traj_mol_center(1:3,1,J,I,IFRAME) = mol_center(1:3)
	    traj_mol_axes(1:3,1:3,J,I,IFRAME) = mol_axes(1:3,1:3)
         END DO	 
      END DO
   END DO

   ! Calculate trajectory of centers in fractional units
   
   CALL TRAJECT_CART_2_FRACT (nframes, traj_mol_center, sss)

   ! Initialize histograms (2D --> 5 related to the number of faces of the Voronoi polyhedra)
   
   number_points = int(rmax/dr) + 1
   IF ( ALLOCATED(norm1) ) DEALLOCATE(norm1)
   IF ( ALLOCATED(ocf) ) DEALLOCATE(ocf)
   ALLOCATE ( norm1 (5, number_points ) ); norm1 = 0
   ALLOCATE ( ocf ( 5, 4, number_points ) ); ocf = 0.0
   
   number_th = INT(180./dtheta) + 1
   IF (number_th > 20) PRINT *,'Output file will contain more than one row per r value. Number of angles = ',number_th
   IF ( ALLOCATED(hist_distth) )      DEALLOCATE(hist_distth)
   IF ( ALLOCATED(hist_distth_sa) )   DEALLOCATE(hist_distth_sa)
   IF ( ALLOCATED(hist_distth_2) )    DEALLOCATE(hist_distth_2)
   IF ( ALLOCATED(hist_distth_sa_2) ) DEALLOCATE(hist_distth_sa_2)
   ALLOCATE ( hist_distth ( 5, number_points, number_th ) );      hist_distth = 0.
   ALLOCATE ( hist_distth_sa ( 5, number_points, number_th ) );   hist_distth_sa = 0.
   ALLOCATE ( hist_distth_2 ( 5, number_points, number_th ) );    hist_distth_2 = 0.
   ALLOCATE ( hist_distth_sa_2 ( 5, number_points, number_th ) ); hist_distth_sa_2 = 0.
      
   number_r = INT(rmax) + 1
   IF (number_r > 20) PRINT *,'Output file will contain more than one row per theta value. Number of distances = ',number_r
   IF ( ALLOCATED(norm2) )            DEALLOCATE(norm2)
   IF ( ALLOCATED(hist_thdist) )      DEALLOCATE(hist_thdist)
   IF ( ALLOCATED(hist_thdist_sa) )   DEALLOCATE(hist_thdist_sa)
   IF ( ALLOCATED(hist_thdist_2) )    DEALLOCATE(hist_thdist_2)
   IF ( ALLOCATED(hist_thdist_sa_2) ) DEALLOCATE(hist_thdist_sa_2)
   ALLOCATE ( norm2 ( 5, number_r ) ); norm2 = 0
   ALLOCATE ( hist_thdist ( 5, 181, number_r ) );    hist_thdist = 0.
   ALLOCATE ( hist_thdist_sa ( 5, 181, number_r ) ); hist_thdist_sa = 0.
   ALLOCATE ( hist_thdist_2 ( 5, 181, number_r ) );  hist_thdist_2   = 0.
   ALLOCATE ( hist_thdist_sa_2( 5, 181, number_r) );  hist_thdist_sa_2 = 0.

   nprinted = 0

   ! Read Voronoi files
   
   DO IFRAME = nfirst, nlast 

      WRITE (frame_number,'(I4.4)') IFRAME
      vol_file = voro_file(1:LEN_TRIM(voro_file))//frame_number//'.xyz.vol'
      UNIT = NEXT_UNIT ( )
      OPEN ( UNIT, FILE = vol_file, ACTION = "READ", STATUS = "OLD", IOSTAT = IOSTAT )
      CALL PUSH_UNIT ( UNIT )

      DO WHILE (.NOT.end) 

         CALL GET_LINE (end)
         IF (.NOT.end) THEN
	    CALL GET_INTEGER ( index )
	    IF (index > nmolecules) STOP 'Increase number of molecules'
	    CALL GET_REAL ( x )
	    CALL GET_REAL ( y )
	    CALL GET_REAL ( z )
	    CALL GET_INTEGER ( vertices )
	    CALL GET_INTEGER ( edges )
	    CALL GET_INTEGER ( faces )
	    CALL GET_REAL ( volume )
            DO I = 1, faces
	       CALL GET_INTEGER ( neighbour(I) )
	    END DO

            ! Orientation between index & neighbour(I)

            !print *, 'index = ', index
            DO I = 1, faces
	       !print *, 'face no. = ', I
               
	       IF (faces <= 12) THEN
	          index_face = 1
	       ELSE IF (faces == 13) THEN
	          index_face = 2
	       ELSE IF (faces == 14) THEN
	          index_face = 3
	       ELSE IF (faces == 15) THEN
	          index_face = 4
	       ELSE
	          index_face = 5
	       END IF	  	  
	       	    
	       sdist= sss(1:3,1,index,1,IFRAME) - sss(1:3,1,neighbour(I),1,IFRAME)
	       sdist = sdist - ANINT(sdist)
               rx = sdist(1)*frame_cell(1,1,IFRAME) + sdist(2)*frame_cell(2,1,IFRAME) + sdist(3)*frame_cell(3,1,IFRAME)
               ry = sdist(1)*frame_cell(1,2,IFRAME) + sdist(2)*frame_cell(2,2,IFRAME) + sdist(3)*frame_cell(3,2,IFRAME)
               rz = sdist(1)*frame_cell(1,3,IFRAME) + sdist(2)*frame_cell(2,3,IFRAME) + sdist(3)*frame_cell(3,3,IFRAME)
               r = SQRT( rx*rx + ry*ry + rz*rz )
		     
	       IF (r.le.rmax) THEN
	    
                  !Print some close neighbours
		  IF (r.le.5.0) THEN
		     nprinted(index_face) = nprinted(index_face) + 1
		     !print *, index_face, nprinted(index_face), r
		     IF (nprinted(index_face).le.max_print) THEN
	                WRITE (index1_name,"(I2.2)") index_face
	                WRITE (index2_name,"(I2.2)") nprinted(index_face)
                        UNIT = NEXT_UNIT ( )	
                        OPEN ( UNIT, FILE = 'Check_'//index1_name//'_'//index2_name//'.pdb', ACTION = "WRITE", &
			       STATUS = "UNKNOWN", IOSTAT = IOSTAT )
                        DO IAUX = 1,numatoms(1)
	                   WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', IAUX, atomname(IAUX, 1), 'MOL', &
	                   1, rrr(1:3, IAUX, index, 1, IFRAME), 0.0, 0.0, 'XXX' 
			END DO
                        DO IAUX = 1,numatoms(1)
	                   WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', numatoms(1)+IAUX, &
			   atomname(IAUX, 1), 'MOL', 2, rrr(1:3, IAUX, neighbour(I), 1, IFRAME), 0.0, 0.0, 'XXX' 
			END DO
	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+1, 'CM ', 'MOL', &
	                   1, traj_mol_center(1:3, 1, index, 1, IFRAME), 0.0, 0.0, 'XXX' 
	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+2, 'CM ', 'MOL', &
	                   2, traj_mol_center(1:3, 1, neighbour(I), 1, IFRAME), 0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+3, 'X  ', 'MOL', &
	                   1, traj_mol_center(1:3, 1, index, 1, IFRAME)+traj_mol_axes(1:3, 1, index, 1, IFRAME), 0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+4, 'Y  ', 'MOL', &
	                   1, traj_mol_center(1:3, 1, index, 1, IFRAME)+traj_mol_axes(1:3, 2, index, 1, IFRAME), 0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+5, 'Z  ', 'MOL', &
	                   1, traj_mol_center(1:3, 1, index, 1, IFRAME)+traj_mol_axes(1:3, 3, index, 1, IFRAME), 0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+6, 'X  ', 'MOL', &
	                   2, traj_mol_center(1:3, 1, neighbour(I), 1, IFRAME)+traj_mol_axes(1:3, 1, neighbour(I), 1, IFRAME), &
			   0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+7, 'Y  ', 'MOL', &
	                   2, traj_mol_center(1:3, 1, neighbour(I), 1, IFRAME)+traj_mol_axes(1:3, 2, neighbour(I), 1, IFRAME), &
			   0.0, 0.0, 'XXX' 
 	                WRITE (UNIT,'(A6,I5,2x,A3,1x,A3,2x,I4,4x,3F8.3,2F6.2,6x,A4)') 'ATOM  ', 2*numatoms(1)+8, 'Z  ', 'MOL', &
	                   2, traj_mol_center(1:3, 1, neighbour(I), 1, IFRAME)+traj_mol_axes(1:3, 3, neighbour(I), 1, IFRAME), &
			   0.0, 0.0, 'XXX' 
			CLOSE (UNIT)   
	             ENDIF 
		  ENDIF

         	  bin = INT(r/dr) + 1
        	  norm1(index_face, bin) = norm1(index_face, bin) + 1
        	  dot = DOT_PRODUCT ( traj_mol_axes(1:3,1,index,1,IFRAME), traj_mol_axes(1:3,1,neighbour(I),1,IFRAME) )
        	  ocf(index_face, 1,bin) = ocf(index_face, 1,bin) + dot
        	  dot = DOT_PRODUCT ( traj_mol_axes(1:3,2,index,1,IFRAME), traj_mol_axes(1:3,2,neighbour(I),1,IFRAME) )
        	  ocf(index_face, 2,bin) = ocf(index_face, 2,bin) + dot
        	  dot = DOT_PRODUCT ( traj_mol_axes(1:3,3,index,1,IFRAME), traj_mol_axes(1:3,3,neighbour(I),1,IFRAME) )
        	  ocf(index_face, 3,bin) = ocf(index_face, 3,bin) + dot
        	  ocf(index_face, 4,bin) = ocf(index_face, 4,bin) + abs(dot)

		  theta3 = ACOS(dot)*180.0/pi
		  binth = INT(theta3/dtheta) + 1
		  hist_distth(index_face, bin, binth) = hist_distth(index_face, bin, binth) + 1. !full histogram

		  !weight each point by 1/sin(theta) to take into account solid angle
		  sinth = SIN(ACOS(dot))
		  IF (sinth < 1.0e-3) sinth = 1.0e-3
		  hist_distth_sa(index_face, bin, binth) = hist_distth_sa(index_face, bin, binth) + (1./sinth) 

		  binr = INT(r) + 1
		  norm2(index_face,binr) = norm2(index_face,binr) + 1 
		  binth = INT(theta3/1.0) + 1
		  hist_thdist(index_face, binth, binr) = hist_thdist(index_face, binth, binr) + 1. !full histogram

		  !weight each point by 1/sin(theta) to take into account solid angle
		  hist_thdist_sa(index_face, binth, binr) = hist_thdist_sa(index_face, binth, binr) + (1./sinth)

               END IF

            END DO

         END IF

      END DO
      CLOSE ( UNIT )
      CALL POP_UNIT 
      end = .FALSE.

   END DO
   
   !Normalize histograms by number of frames and number of molecules
   
   hist_distth = hist_distth / REAL(nframes) / REAL(nummols(1))
   hist_thdist = hist_thdist / REAL(nframes) / REAL(nummols(1))
   hist_distth_sa = hist_distth_sa / REAL(nframes) / REAL(nummols(1))
   hist_thdist_sa = hist_thdist_sa / REAL(nframes) / REAL(nummols(1))
   
   !Normalize histograms by number of pairs found at each r
   
   DO IFACE = 1,5
   DO I = 1,number_points
      IF (norm1(IFACE,I) > 0) THEN
         DO J = 1,number_th
            hist_distth_2(IFACE,I,J) = hist_distth(IFACE,I,J) * REAL(nummols(1)) / REAL(norm1(IFACE,I))
            hist_distth_sa_2(IFACE,I,J) = hist_distth_sa(IFACE,I,J) * REAL(nummols(1)) / REAL(norm1(IFACE,I))
         END DO
      END IF
   END DO
   END DO

   DO IFACE = 1,5
   DO I = 1,181
      DO J = 1,number_r 
         IF (norm2(IFACE,J) > 0) THEN 
            hist_thdist_2(IFACE,I,J) = hist_thdist(IFACE,I,J) * REAL(nummols(1)) / norm2(IFACE,J)
            hist_thdist_sa_2(IFACE,I,J) = hist_thdist_sa(IFACE,I,J) * REAL(nummols(1)) / norm2(IFACE,J)
	 END IF
      END DO
   END DO
   END DO
   
   IF (normalize_maximum) THEN
     
      norm_value_1 = maxval(hist_distth) / 9.999
      hist_distth = hist_distth / norm_value_1
      norm_value_2 = maxval(hist_thdist) / 9.999
      hist_thdist = hist_thdist / norm_value_2
      norm_value_3 = maxval(hist_distth_sa) / 9.999
      hist_distth_sa = hist_distth_sa / norm_value_3
      norm_value_4 = maxval(hist_thdist_sa) / 9.999
      hist_thdist_sa = hist_thdist_sa / norm_value_4
      norm_value_5 = maxval(hist_distth_2) / 9.999
      hist_distth_2 = hist_distth_2 / norm_value_5
      norm_value_6 = maxval(hist_thdist_2) / 9.999
      hist_thdist_2 = hist_thdist_2 / norm_value_6
      norm_value_7 = maxval(hist_distth_sa_2) / 9.999
      hist_distth_sa_2 = hist_distth_sa_2 / norm_value_7
      norm_value_8 = maxval(hist_thdist_sa_2) / 9.999
      hist_thdist_sa_2 = hist_thdist_sa_2 / norm_value_8
   END IF
    
   ! Write results

   DO IFACE = 1, 5
   
   WRITE (iface_number,'(I1.1)') IFACE

   !Orientational correlation function vs r
   
   UNIT1 = NEXT_UNIT ( )
   OPEN ( UNIT1, FILE = 'iface'//iface_number//'_'//output_file_1, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT1,'("# Orientational correlation functions")')
   WRITE (UNIT1,'("#",3x,"r/A",3x,"OCF 1",7x,"OCF 2",6x,"OCF 3",6x,"|OCF 3|")')

   !Histograms of population as a function of r (for different angle values)
   
   !Full
   UNIT2 = NEXT_UNIT ( )
   OPEN ( UNIT2, FILE = 'iface'//iface_number//'_'//'full_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT2,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT2,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT2,'("# Not normalized by solid angle")')
   IF (normalize_maximum) WRITE (UNIT2,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_1
   WRITE (UNIT2,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Weighted by solid angle considered as sin(theta)
   UNIT3 = NEXT_UNIT ( )
   OPEN ( UNIT3, FILE = 'iface'//iface_number//'_'//'wsin_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT3,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT3,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT3,'("# Solid angle taken into account by weighting each point by 1/sin(theta).")')
   IF (normalize_maximum) WRITE (UNIT3,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_3
   WRITE (UNIT3,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Normalized by number of pairs found at each r
   UNIT4 = NEXT_UNIT ( )
   OPEN ( UNIT4, FILE = 'iface'//iface_number//'_'//'norm_'//output_file_2, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT4,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT4,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT4,'("# Not normalized by solid angle")')
   IF (normalize_maximum) WRITE (UNIT4,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_5
   WRITE (UNIT4,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta
   
   !Normalized by number of pairs found at each r and weighted by solid angle considered as sin(theta)
   UNIT5 = NEXT_UNIT ( )
   OPEN ( UNIT5, FILE = 'iface'//iface_number//'_'//'normwsin_'//output_file_2, ACTION = "WRITE", IOSTAT = IOSTAT )
   WRITE (UNIT5,'("# Histogram of distribution of angles vs r")')
   WRITE (UNIT5,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT5,'("# Solid angle taken into account by weighting each point by 1/sin(theta)")')
   IF (normalize_maximum) WRITE (UNIT5,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_7
   WRITE (UNIT5,'("# r/A --- Angles 0-180 with dtheta = ",F6.2)') dtheta


   !Histograms of population as a function of angle (for different r values)
   
   !Full
   UNIT6 = NEXT_UNIT ( )
   OPEN ( UNIT6, FILE = 'iface'//iface_number//'_'//'full_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT6,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT6,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT6,'("# Not normalized by solid angle.")')
   IF (normalize_maximum) WRITE (UNIT6,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_2
   WRITE (UNIT6,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Weighted by solid angle considered as sin(theta)
   UNIT7 = NEXT_UNIT ( )
   OPEN ( UNIT7, FILE = 'iface'//iface_number//'_'//'wsin_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT7,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT7,'("# Divided by number of frames and number of molecules.")')
   WRITE (UNIT7,'("# Solid angle taken into account by weighting each point by 1/sin(theta).")')
   IF (normalize_maximum) WRITE (UNIT7,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_4
   WRITE (UNIT7,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Normalized by number of pairs found at each r
   UNIT8 = NEXT_UNIT ( )
   OPEN ( UNIT8, FILE = 'iface'//iface_number//'_'//'norm_'//output_file_3, ACTION = "WRITE", STATUS = "UNKNOWN", IOSTAT = IOSTAT )
   WRITE (UNIT8,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT8,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT8,'("# Not normalized by solid angle.")')
   IF (normalize_maximum) WRITE (UNIT8,'("# Divided by ",G10.4)') norm_value_6
   WRITE (UNIT8,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
   
   !Normalized by number of pairs found at each r and weighted by solid angle considered as sin(theta)
   UNIT9 = NEXT_UNIT ( )
   OPEN ( UNIT9, FILE = 'iface'//iface_number//'_'//'normwsin_'//output_file_3, ACTION = "WRITE", IOSTAT = IOSTAT )
   WRITE (UNIT9,'("# Histogram of distribution of distances vs angle")')
   WRITE (UNIT9,'("# Divided by number of frames and number of pairs found at each r.")')
   WRITE (UNIT9,'("# Solid angle taken into account by weighting each point by 1/sin(theta)")')
   IF (normalize_maximum) WRITE (UNIT9,'("# Divided by ",G10.4," so maximum is 9.999")') norm_value_8
   WRITE (UNIT9,'("# Theta --- Distances 0-Rmax with dr = 1.0 A")') 
    
   DO I = 1,number_points
      
      xa = REAL(I-1)*dr
      xb = xa + dr
      xx = 0.5 * (xa + xb)

      IF (norm1(IFACE,I) > 0) THEN
         out1 = ocf(IFACE,1,I) / REAL(norm1(IFACE,I))
         out2 = ocf(IFACE,2,I) / REAL(norm1(IFACE,I))
         out3 = ocf(IFACE,3,I) / REAL(norm1(IFACE,I))
         out4 = ocf(IFACE,4,I) / REAL(norm1(IFACE,I))
      ELSE
         out1 = 0.0d0
         out2 = 0.0d0
         out3 = 0.0d0
         out4 = 0.0d0
      END IF      

      WRITE (UNIT1,"(1x,f7.3,5(2x,f10.6))") xx, out1, out2, out3, out4

      WRITE (UNIT2,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth(IFACE,I,J), J=1,number_th)
      WRITE (UNIT3,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_sa(IFACE,I,J), J=1,number_th)
      WRITE (UNIT4,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_2(IFACE,I,J), J=1,number_th)
      WRITE (UNIT5,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_distth_sa_2(IFACE,I,J), J=1,number_th)


   END DO
   
   DO I = 1,180

      xa = REAL(I-1)*1.0
      xb = xa + 1.0
      xx = 0.5 * (xa + xb)
      WRITE (UNIT6,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist(IFACE,I,J), J=1,number_r)
      WRITE (UNIT7,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_sa(IFACE,I,J), J=1,number_r)
      WRITE (UNIT8,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_2(IFACE,I,J), J=1,number_r)
      WRITE (UNIT9,"(1x,f7.3,1x,20(1x,F5.3))") xx, (hist_thdist_sa_2(IFACE,I,J), J=1,number_r)

   END DO

   CLOSE (UNIT1)
   CLOSE (UNIT2)
   CLOSE (UNIT3)
   CLOSE (UNIT4)
   CLOSE (UNIT5)
   CLOSE (UNIT6)
   CLOSE (UNIT7)
   CLOSE (UNIT8)
   CLOSE (UNIT9)

   END DO
   
END PROGRAM VORONOI_ORIENTATION_ANALYSIS
