!
! Reads a trajectory file and writes a PDB file of each selected frame
!
! 	Reads:
!	1. HISTORY (+FIELD) from DL_POLY
!
!	Writes:
!	1. A series of PDB files
!
!  Last change: 08/01/2010
!

PROGRAM WRITE_TRAJECTFRAME_PDB	

   USE DEFINITIONS,          ONLY : LINE_LENGTH, DP
   USE IO_UNITS,             ONLY : INPUT, OUTPUT
   USE GENERAL_VARIABLES
   USE TRAJECTORY_VARIABLES, ONLY : pbc_jumps
   USE FORCEFIELD_READ,      ONLY : FIELD_DLPOLY_READ
   USE TRAJECTORY_READWRITE, ONLY : HISTORY_DLPOLY_READ
   USE TRAJECTORY_UTILS,     ONLY : TRAJECT_UNWRAP, TRAJECT_JOIN_MOLECULES
   USE CONFIG_READWRITE,     ONLY : CONFIG_PDB_WRITE
   
   IMPLICIT NONE

   CHARACTER ( LEN = LINE_LENGTH)      :: traject_file, ff_file, output_file, pdb_file
   CHARACTER ( LEN = 4)                :: frame_number
   INTEGER                             :: I, IFRAME, nfirst, nlast, ngap

   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:,:) :: traject_r
   REAL ( KIND = DP ), ALLOCATABLE, DIMENSION(:,:,:,:)   :: config_pos


   ! Get input
   
   WRITE (OUTPUT,'("Trajectory (HISTORY) file name?")')
   READ (INPUT,'(A)') traject_file
   WRITE (OUTPUT,'("FIELD file (enter 0 if file not present)?")')
   READ (INPUT,'(A)') ff_file
   WRITE (OUTPUT,'("Number of first frame to write as a PDB file?")')
   READ (INPUT,*) nfirst
   WRITE (OUTPUT,'("Number of last frame to write as a PDB file?")')
   READ (INPUT,*) nlast
   WRITE (OUTPUT,'("Selected gap between PDB files to write?")')
   READ (INPUT,*) ngap
   WRITE (OUTPUT,'("Name of output file (without extension .pdb)?")')
   READ (INPUT,'(A)') output_file
   WRITE (OUTPUT,*)
   

   ! Get system information
   
   IF (ff_file(1:1) == "0") THEN
      WRITE (OUTPUT,'("Number of different molecular species?")')
      READ (INPUT,*) numspecies
      IF ( ALLOCATED(nummols) ) DEALLOCATE(nummols)
      IF ( ALLOCATED(numatoms) ) DEALLOCATE(numatoms)
      ALLOCATE ( nummols(numspecies), numatoms(numspecies) )
      DO I = 1,numspecies
	 WRITE (OUTPUT,'("Number of molecules of type ",I3,"? ")', ADVANCE='NO') I
	 READ (INPUT,*) nummols(I) 
	 WRITE (OUTPUT,'("Number of atoms in this molecule? ")', ADVANCE='NO')
	 READ (INPUT,*) numatoms(I) 
      END DO
   ELSE
      WRITE (OUTPUT,'("Reading information from FIELD file")')
      CALL FIELD_DLPOLY_READ (ff_file)
   END IF 

   
   PRINT *,'Reading trajectory ...'
   CALL HISTORY_DLPOLY_READ ( traject_file, nlast, traject_r )
   PRINT *,'Finished reading trajectory!'
   IF (pbc_jumps) CALL TRAJECT_UNWRAP ( nlast, traject_r )  
   CALL TRAJECT_JOIN_MOLECULES ( nlast, traject_r )
   
   ALLOCATE ( config_pos ( 3, maxval(numatoms), maxval(nummols), numspecies ) ); config_pos = 0.0
   DO IFRAME = nfirst, nlast, ngap 
      config_pos ( 1:3, :, :, : ) = traject_r (1:3, :, :, :, IFRAME)
      WRITE (frame_number,'(I4.4)') IFRAME
      pdb_file = output_file(1:LEN_TRIM(output_file))//frame_number//'.pdb'
      CALL CONFIG_PDB_WRITE ( pdb_file, config_pos)
   END DO
   
END PROGRAM WRITE_TRAJECTFRAME_PDB
